#include "networkbroadcastdialog.h"

#include <QUdpSocket>
#include <QLabel>
#include <QDebug>
#include <QVBoxLayout>

NetworkBroadcastDialog::NetworkBroadcastDialog(QWidget *parent) :
    QDialog(parent),
    _repeatCount(0),
    _port(-1),
    _isUnicode(false)
{
    QLabel *pLbl = new QLabel;
    pLbl->setText(tr("Searching,please wait... ..."));

    QVBoxLayout *pVlt = new QVBoxLayout;
    pVlt->addWidget(pLbl);

    setLayout(pVlt);

    setWindowTitle(tr("Quick search device"));

    // 每隔2秒发送一次广播
    _searchTimer.setInterval(2000);
    connect(&_searchTimer,SIGNAL(timeout()),this,SLOT(slotOnSearch()));

    // 创建广播发送对象 QUdpSocket
    _p_broadcastSocket = new QUdpSocket;
    _p_broadcastSocket->bind(20202,QUdpSocket::ShareAddress);
    connect(_p_broadcastSocket,SIGNAL(readyRead()),this,SLOT(slotOnBroadcastDatagram()));
}

NetworkBroadcastDialog::~NetworkBroadcastDialog()
{
    delete _p_broadcastSocket;
}

void NetworkBroadcastDialog::finishSearch(int result)
{
    _searchTimer.stop();
    done(result);
}

int NetworkBroadcastDialog::exec()
{
    _searchTimer.start();
    return QDialog::exec();
}

// 发送广播
void NetworkBroadcastDialog::slotOnSearch()
{
    _repeatCount++;

    // 如果发送广播次数超过五次则停止搜索
    if(_repeatCount > 5) {
        finishSearch(0);
        return;
    }

    // 发送广播
    QByteArray datagram("<?DEVICE>");
    _p_broadcastSocket->writeDatagram(datagram.data(),datagram.size(),QHostAddress::Broadcast,10101);
}

// 收到回应
void NetworkBroadcastDialog::slotOnBroadcastDatagram()
{
    QByteArray recvBuffer;
    // 读取回应的全部内容
    while(_p_broadcastSocket->hasPendingDatagrams()) {
        QByteArray datagram;
        datagram.resize(_p_broadcastSocket->pendingDatagramSize());
        _p_broadcastSocket->readDatagram(datagram.data(),datagram.size());

        recvBuffer += datagram;
    }

    // 判断回应内容是否符合协议
    readDeviceFromDatagram(recvBuffer);
}

void NetworkBroadcastDialog::readDeviceFromDatagram(const QByteArray &bytes)
{
    qDebug() << "NetworkBroadcastDialog::readDeviceFromDatagram:: " << bytes;

    // frame format
    // <DEVICE,NAME,IP,PORT,ENCODE,ID>

    // 协议格式应为<DEVICE,板卡名称,板卡IP地址,板卡端口号,板卡通讯模式,板卡ID>
    QByteArray recvBytes = bytes.trimmed();
    if(!recvBytes.startsWith("<") || !recvBytes.endsWith(">")) {
        return;
    }

    // remove "<" and ">"
    recvBytes = recvBytes.mid(1,recvBytes.size() - 2);

    // 以“，”为分隔符，分解内容
    QStringList list = QString(recvBytes).split(",",QString::SkipEmptyParts);
    if(list.isEmpty()) {
        return;
    }

    // 分解出的内容个数不为6个，则不满足协议格式
    if(list.size() != 6) {
        return;
    }

    // 分解出的内容的首个内容不为“DEVICE”，则不满足协议格式
    if(list.first() != "DEVICE") {
        return;
    }

    // 根据协议内容，可以得到
    _deviceName = list[1]; // 板卡名称
    _ipAddr = list[2]; // 板卡IP地址
    _port = list[3].toInt(); // 板卡端口号
    _isUnicode = (list[4] == "UNICODE"); // 板卡通讯模式
    _deviceID = list[5]; // 板卡ID

    // 成功搜索到板卡
    finishSearch(1);
}
