#include "serialportwidget.h"

#include <QPushButton>
#include <QComboBox>
#include <QLabel>
#include <QGridLayout>
#include <QSerialPort>
#include <QSerialPortInfo>

SerialPortWidget::SerialPortWidget(QWidget *parent) : QWidget(parent)
{
    // 单次触发计时器，用于读取网口超时计时
    _timer.setSingleShot(true);
    _timer.setInterval(100);
    connect(&_timer,SIGNAL(timeout()),this,SLOT(slotOnTimeout()));


    // 创建控件
    QLabel *pPortLbl = new QLabel(tr("Port"));
    pPortLbl->setFixedWidth(80);

    QList<QSerialPortInfo> availiablePorts = QSerialPortInfo::availablePorts();
    _p_portCbx = new QComboBox;
    for(int i = 0;i < availiablePorts.size();i++) {
        _p_portCbx->addItem(availiablePorts[i].portName());
    }
    _p_portCbx->setFixedWidth(130);

    QLabel *pBaudrateLbl = new QLabel(tr("Baudrate"));
    pBaudrateLbl->setFixedWidth(80);

    _p_baudrateCbx = new QComboBox;
    _p_baudrateCbx->addItem("9600",(int)QSerialPort::Baud9600);
    _p_baudrateCbx->addItem("19200",(int)QSerialPort::Baud19200);
    _p_baudrateCbx->addItem("38400",(int)QSerialPort::Baud38400);
    _p_baudrateCbx->addItem("57600",(int)QSerialPort::Baud57600);
    _p_baudrateCbx->addItem("115200",(int)QSerialPort::Baud115200);
    _p_baudrateCbx->setCurrentIndex(4);
    _p_baudrateCbx->setFixedWidth(130);

    QLabel *pDatabitLbl = new QLabel(tr("Data bit"));
    pDatabitLbl->setFixedWidth(80);

    _p_databitCbx = new QComboBox;
    _p_databitCbx->addItem("6",(int)QSerialPort::Data6);
    _p_databitCbx->addItem("7",(int)QSerialPort::Data7);
    _p_databitCbx->addItem("8",(int)QSerialPort::Data8);
    _p_databitCbx->setCurrentIndex(2);
    _p_databitCbx->setFixedWidth(130);

    QLabel *pParitybitLbl = new QLabel(tr("Parity"));
    pParitybitLbl->setFixedWidth(80);

    _p_paritybitCbx = new QComboBox;
    _p_paritybitCbx->addItem(tr("None"),(int)QSerialPort::NoParity);
    _p_paritybitCbx->addItem(tr("Even"),(int)QSerialPort::EvenParity);
    _p_paritybitCbx->addItem(tr("Odd"),(int)QSerialPort::OddParity);
    _p_paritybitCbx->setCurrentIndex(0);
    _p_paritybitCbx->setFixedWidth(130);

    QLabel *pStopbitLbl = new QLabel(tr("Stop bit"));
    pStopbitLbl->setFixedWidth(80);

    _p_stopbitCbx = new QComboBox;
    _p_stopbitCbx->addItem("1",(int)QSerialPort::OneStop);
    _p_stopbitCbx->addItem("2",(int)QSerialPort::TwoStop);
    _p_stopbitCbx->setCurrentIndex(0);
    _p_stopbitCbx->setFixedWidth(130);

    QLabel *pModeLbl = new QLabel(tr("Mode"));
    pModeLbl->setFixedWidth(80);

    _p_modeCbx = new QComboBox;
    _p_modeCbx->addItems(QStringList() << tr("Ascii") << tr("Unicode"));
    _p_modeCbx->setCurrentIndex(0);
    _p_modeCbx->setFixedWidth(130);
    connect(_p_modeCbx,SIGNAL(currentIndexChanged(int)),this,SLOT(slotOnModeChanged()));

    _p_openBtn = new QPushButton(tr("Open"));
    _p_openBtn->setFixedWidth(130);
    connect(_p_openBtn,SIGNAL(clicked()),this,SLOT(doOpen()));

    _p_closeBtn = new QPushButton(tr("Close"));
    _p_closeBtn->setFixedWidth(130);
    connect(_p_closeBtn,SIGNAL(clicked()),this,SLOT(doClose()));


    // 控件布局
    QGridLayout *pGlt = new QGridLayout;
    pGlt->addWidget(pPortLbl,0,0,1,1);
    pGlt->addWidget(_p_portCbx,0,1,1,1);

    pGlt->addWidget(pBaudrateLbl,1,0,1,1);
    pGlt->addWidget(_p_baudrateCbx,1,1,1,1);

    pGlt->addWidget(pDatabitLbl,2,0,1,1);
    pGlt->addWidget(_p_databitCbx,2,1,1,1);

    pGlt->addWidget(pParitybitLbl,3,0,1,1);
    pGlt->addWidget(_p_paritybitCbx,3,1,1,1);

    pGlt->addWidget(pStopbitLbl,4,0,1,1);
    pGlt->addWidget(_p_stopbitCbx,4,1,1,1);

    pGlt->addWidget(pModeLbl,5,0,1,1);
    pGlt->addWidget(_p_modeCbx,5,1,1,1);

    pGlt->addWidget(_p_openBtn,6,1,1,1);

    pGlt->addWidget(_p_closeBtn,7,1,1,1);

    setLayout(pGlt);

    // 创建串口对象
    _p_serialport = new QSerialPort(this);
    connect(_p_serialport,SIGNAL(readyRead()),this,SLOT(doRead()));
}

SerialPortWidget::~SerialPortWidget()
{
    delete _p_serialport;
}

bool SerialPortWidget::isAvailable() const
{
    return isVisible();
}

bool SerialPortWidget::canRead() const
{
    return (_p_serialport->isOpen());
}

bool SerialPortWidget::canWrite() const
{
    return (_p_serialport->isOpen());
}

void SerialPortWidget::updateState()
{
    // 根据串口打开/关闭修改控件使能状态
    bool isOpened = _p_serialport->isOpen();
    _p_portCbx->setEnabled(!isOpened);
    _p_baudrateCbx->setEnabled(!isOpened);
    _p_databitCbx->setEnabled(!isOpened);
    _p_stopbitCbx->setEnabled(!isOpened);
    _p_paritybitCbx->setEnabled(!isOpened);
    _p_openBtn->setEnabled(!isOpened);
}

void SerialPortWidget::slotOnModeChanged()
{
    _protocol = _p_modeCbx->currentIndex();
}

// 打开串口
void SerialPortWidget::doOpen()
{
    // 如果串口已打开，则不重复打开，直接返回
    if(_p_serialport->isOpen()) {
        return;
    }

    // 获取本机上所有串口
    QList<QSerialPortInfo> availiablePorts = QSerialPortInfo::availablePorts();

    // 获取串口名控件所选择条目索引
    int index = _p_portCbx->currentIndex();
    if((index < 0) || (index >= availiablePorts.size())) {
        return;
    }

    // 根据控件当前选项，配置串口参数
    _p_serialport->setPort(availiablePorts[index]); // 设置串口名
    _p_serialport->setBaudRate((QSerialPort::BaudRate)_p_baudrateCbx->currentData().toInt()); // 设置波特率
    _p_serialport->setDataBits((QSerialPort::DataBits)_p_databitCbx->currentData().toInt()); // 设置数据位
    _p_serialport->setParity((QSerialPort::Parity)_p_paritybitCbx->currentData().toInt()); // 设置校验位
    _p_serialport->setStopBits((QSerialPort::StopBits)_p_stopbitCbx->currentData().toInt()); // 设置停止位

    // 最后打开串口
    if(!_p_serialport->open(QIODevice::ReadWrite)) {
        return;
    }

    updateState();

    emit connected();
}

// 关闭串口
void SerialPortWidget::doClose()
{
    if(!_p_serialport->isOpen()) {
        return;
    }

    _p_serialport->close();

    updateState();

    emit disconnected();
}

void SerialPortWidget::slotOnTimeout()
{
    if(!_readBytes.isEmpty()) {
        emit readyRead();
    }
}

// 读取串口接收到的数据
void SerialPortWidget::doRead()
{
    if(!_timer.isActive()) {
        // 计时器未在运行时串口接收到数据，则认为是新的一帧数据
        _readBytes = _p_serialport->readAll();
    } else {
        // 计时器在运行时串口接收到数据，则认为是上一帧数据的剩余部分
        _readBytes += _p_serialport->readAll();
    }

    // 每次读取串口接收到的数据后，计时器开始计时，当下次再读取串口接收到的数据时，对计时器状态进行判断
    // 如果下次再读取串口接收到的数据时，计时器仍在运行，则表示计时器间隔时间还没过完，认为新读取的数据是上一帧数据的剩余部分
    // 如果下次再读取串口接收到的数据时，计时器已经结束运行，则表示计时器间隔时间已经过完，认为新读取的数据是新一帧数据
    _timer.start();
}

// 发送数据到板卡
int SerialPortWidget::write(const QByteArray &bytes)
{
    if(!_p_serialport->isOpen()) {
        return -1;
    }

    return _p_serialport->write(bytes);
}
