#include "widget.h"
#include "networkwdiget.h"
#include "serialportwidget.h"

#include <QMainWindow>
#include <QRadioButton>
#include <QtDebug>
#include <QApplication>
#include <QLineEdit>
#include <QGridLayout>
#include <QPushButton>
#include <QLabel>
#include <QSpacerItem>
#include <QTcpSocket>
#include <QMessageBox>
#include <QHostAddress>
#include <QImage>
#include <QFileInfo>5
#include <QDesktopWidget>
#include <QComboBox>
#include <QPlainTextEdit>
#include <QHBoxLayout>
#include <QTime>
#include <QScrollBar>
#include <QDesktopServices>
#include <QTabWidget>
#include <QWidget>
#include <QIntValidator>
#include <QDoubleSpinBox>
#include <QSpinBox>

/*************************** Logo link ********************************/

#define _LOGO_WIDTH		212
#define _LOGO_HEIGHT	60

LabelLink::LabelLink(QWidget *parent) :
    QLabel(parent),
    _clickInfo(false),
    _repeatCount(0)
{
    _prevTime = QTime::currentTime();

    setScaledContents(true);
    setOpenExternalLinks(true);
    setMouseTracking(true);
}

void LabelLink::setEnableClickInfo(bool on)
{
    _clickInfo = on;
}

void LabelLink::mousePressEvent(QMouseEvent *event)
{
    if(!_clickInfo) {
        return;
    }

    int elapse = QTime::currentTime().msecsSinceStartOfDay() - _prevTime.msecsSinceStartOfDay();
    if(elapse > 500) {
        _repeatCount = 0;
    } else {
        _repeatCount++;
    }

    if(_repeatCount >= 10) {
        _repeatCount = 0;
        wchar_t wchs[32] = { 0 };
        wchs[0] = 0x5e7f;wchs[1]=0x5dde;wchs[2]=0x5e02;wchs[3]=0x5fae;
        wchs[4] = 0x5d4c;wchs[5]=0x96f6;wchs[6]=0x58f9;wchs[7]=0x81ea;
        wchs[8] = 0x52a8;wchs[9]=0x5316;wchs[10]=0x79d1;wchs[11]=0x6280;
        wchs[12] = 0x6709;wchs[13]=0x9650;wchs[14]=0x516c;wchs[15]=0x53f8;
        wchs[16] = 0x5236;wchs[17]=0x4f5c;
        QString name = QString::fromWCharArray(wchs);

        wchar_t wtchs[32] = { 0 };
        wtchs[0] = 0x96f6;wtchs[1]=0x58f9;wtchs[2]=0x901a;wtchs[3]=0x8baf;
        wtchs[4] = 0x8c03;wtchs[5]=0x8bd5;wtchs[6]=0x52a9;wtchs[7]=0x624b;
        QString title = QString::fromWCharArray(wtchs);

        QMessageBox::information(this,title,name);
    }

    _prevTime = QTime::currentTime();
}

void LabelLink::resizeEvent(QResizeEvent *event)
{
}

void LabelLink::mouseMoveEvent(QMouseEvent *event)
{
}

static QByteArray toHex(const QByteArray &bytes)
{
    QByteArray hexBytes;
    const char *pBytes = bytes.data();
    for(int i = 0;i < bytes.size();i++) {
        hexBytes += QByteArray(1,(uchar)pBytes[i]).toHex().toUpper() + " ";
    }
    return hexBytes;
}

LineEdit::LineEdit(QWidget *parent) : QLineEdit(parent)
{
}

void LineEdit::focusInEvent(QFocusEvent *e)
{
    QLineEdit::focusInEvent(e);
}

void LineEdit::focusOutEvent(QFocusEvent *e)
{
    QLineEdit::focusOutEvent(e);
}

void LineEdit::inputMethodEvent(QInputMethodEvent *e)
{
    QLineEdit::inputMethodEvent(e);
}

Widget::Widget(QWidget *parent)
    : QWidget(parent)
{
    // 主界面控件创建和布局

    _p_serialportRbn = new QRadioButton(tr("Serialport"));
    _p_serialportRbn->setFixedWidth(80);
    _p_serialportRbn->setChecked(true);
    connect(_p_serialportRbn,SIGNAL(clicked()),this,SLOT(slotOnDeviceChanged()));

    _p_networkRbn = new QRadioButton(tr("Network"));
    _p_networkRbn->setFixedWidth(130);
    connect(_p_networkRbn,SIGNAL(clicked()),this,SLOT(slotOnDeviceChanged()));

    QHBoxLayout *pDeviceHlt = new QHBoxLayout;
    pDeviceHlt->addWidget(_p_serialportRbn);
    pDeviceHlt->addWidget(_p_networkRbn);

    // 创建网口参数设置子界面
    _p_networkWgt = new NetworkWdiget;
    connect(_p_networkWgt,SIGNAL(connected()),this,SLOT(slotOnHasConnected()));
    connect(_p_networkWgt,SIGNAL(disconnected()),this,SLOT(slotOnHasDisconnected()));
    connect(_p_networkWgt,SIGNAL(readyRead()),this,SLOT(doRead()));

    // 创建串口参数设置子界面
    _p_serialportWgt = new SerialPortWidget;
    connect(_p_serialportWgt,SIGNAL(connected()),this,SLOT(slotOnHasConnected()));
    connect(_p_serialportWgt,SIGNAL(disconnected()),this,SLOT(slotOnHasDisconnected()));
    connect(_p_serialportWgt,SIGNAL(readyRead()),this,SLOT(doRead()));


    // 快捷操作选项卡控件
    m_pTabWidget = new QTabWidget(this);

    //normal page
    QLabel *pDocLbl = new QLabel(tr("Document"));
    pDocLbl->setFixedWidth(80);
    _p_docEdit = new QLineEdit();
    _p_docEdit->setFixedWidth(130);

    _p_docSwitchBtn = new QPushButton(tr("Change document"));
    _p_docSwitchBtn->setFixedWidth(130);
    connect(_p_docSwitchBtn,SIGNAL(clicked()),this,SLOT(slotOnSwitchDoc()));

    QLabel *pEntityLbl = new QLabel(tr("Entity name"));
    pEntityLbl->setFixedWidth(80);
    _p_entityNameEdit = new LineEdit();
    _p_entityNameEdit->setFixedWidth(130);

    QLabel *pEntityContentLbl = new QLabel(tr("Content"));
    pEntityContentLbl->setFixedWidth(80);
    _p_entityContentEdit = new LineEdit();
    _p_entityContentEdit->setFixedWidth(130);

    _p_changeBtn = new QPushButton(tr("Change"));
    _p_changeBtn->setFixedWidth(130);
    connect(_p_changeBtn,SIGNAL(clicked()),this,SLOT(slotOnChangeContent()));

    LabelLink *pZOLbl = new LabelLink;
    pZOLbl->setPixmap(QPixmap(":/logo_zo.png"));
    pZOLbl->setFixedSize(48,48);
    pZOLbl->setEnableClickInfo(true);

    _p_startBtn = new QPushButton(tr("Start mark"));
    _p_startBtn->setFixedWidth(130);
    connect(_p_startBtn,SIGNAL(clicked()),this,SLOT(slotOnStart()));

    _p_stopBtn = new QPushButton(tr("Stop mark"));
    _p_stopBtn->setFixedWidth(130);
    connect(_p_stopBtn,SIGNAL(clicked()),this,SLOT(slotOnStop()));

    QVBoxLayout *pPrintVlt = new QVBoxLayout;
    pPrintVlt->addWidget(_p_startBtn);
    pPrintVlt->addWidget(_p_stopBtn);

    QWidget *normalWidget = new QWidget;

    QGridLayout *pGlt = new QGridLayout(normalWidget);

    pGlt->addWidget(pDocLbl,0,0,1,1);
    pGlt->addWidget(_p_docEdit,0,1,1,1);
    pGlt->addWidget(_p_docSwitchBtn,1,1,1,1);
    pGlt->addItem(new QSpacerItem(100,30),2,1,1,1);
    pGlt->addWidget(pEntityLbl,3,0,1,1);
    pGlt->addWidget(_p_entityNameEdit,3,1,1,1);
    pGlt->addWidget(pEntityContentLbl,4,0,1,1);
    pGlt->addWidget(_p_entityContentEdit,4,1,1,1);
    pGlt->addWidget(_p_changeBtn,5,1,1,1);
    pGlt->addItem(new QSpacerItem(100,30),6,1,1,1);
    pGlt->addWidget(pZOLbl,7,0,2,1,Qt::AlignLeft);
    pGlt->addLayout(pPrintVlt,7,1,2,1);

    m_pTabWidget->addTab(normalWidget,tr("General"));

    //ad-1
    QLabel *pTextLbl = new QLabel(tr("Test func"));
    pTextLbl->setFixedWidth(80);
    m_pTestEdit = new QLineEdit;
    m_pTestEdit->setFixedWidth(130);
    m_pTestBtn = new QPushButton(tr("Test"));
    m_pTestBtn->setFixedWidth(130);
    connect(m_pTestBtn,SIGNAL(clicked()),this,SLOT(slotOnTest()));

    QLabel *pSpeedLbl = new QLabel(tr("Mark speed"));
    pSpeedLbl->setFixedWidth(80);
    m_pSpeedEdit = new QLineEdit;
    m_pSpeedEdit->setFixedWidth(130);
    m_pSpeedEdit->setValidator(new QIntValidator(0, 10000, this));
    QLabel *pPowerLbl = new QLabel(tr("Power"));
    pPowerLbl->setFixedWidth(80);
    m_pPowerEdit = new QLineEdit;
    m_pPowerEdit->setFixedWidth(130);
    m_pPowerEdit->setValidator(new QIntValidator(0, 100, this));
    QLabel *pFreqLbl = new QLabel(tr("Frequency"));
    pFreqLbl->setFixedWidth(80);
    m_pFreqEdit = new QLineEdit;
    m_pFreqEdit->setFixedWidth(130);
    m_pFreqEdit->setValidator(new QIntValidator(0, 200, this));
    m_pChangedBtn = new QPushButton(tr("Modify mark params"));
    m_pChangedBtn->setFixedWidth(130);
    connect(m_pChangedBtn,SIGNAL(clicked()),this,SLOT(slotOnChangePen()));

    m_pMarkCountBtn = new QPushButton(tr("Query finished count"));
    m_pMarkCountBtn->setFixedWidth(130);
    connect(m_pMarkCountBtn,SIGNAL(clicked()),this,SLOT(slotOnReadMarkCount()));
    m_pMarkTimeBtn = new QPushButton(tr("Query mark time"));
    m_pMarkTimeBtn->setFixedWidth(130);
    connect(m_pMarkTimeBtn,SIGNAL(clicked()),this,SLOT(slotOnReadMarkTime()));

    QWidget *ad1Widget = new QWidget;

    QGridLayout *pGltAd1 = new QGridLayout(ad1Widget);
    pGltAd1->addWidget(pTextLbl,0,0,1,1);
    pGltAd1->addWidget(m_pTestEdit,0,1,1,1);
    pGltAd1->addWidget(m_pTestBtn,1,1,1,1);
    pGltAd1->addWidget(pSpeedLbl,2,0,1,1);
    pGltAd1->addWidget(m_pSpeedEdit,2,1,1,1);
    pGltAd1->addWidget(pPowerLbl,3,0,1,1);
    pGltAd1->addWidget(m_pPowerEdit,3,1,1,1);
    pGltAd1->addWidget(pFreqLbl,4,0,1,1);
    pGltAd1->addWidget(m_pFreqEdit,4,1,1,1);
    pGltAd1->addWidget(m_pChangedBtn,5,1,1,1);
    pGltAd1->addItem(new QSpacerItem(100,30),6,1,1,1);
    pGltAd1->addWidget(m_pMarkCountBtn,7,1,1,1);
    pGltAd1->addWidget(m_pMarkTimeBtn,8,1,1,1);

    m_pTabWidget->addTab(ad1Widget,tr("Advance 1"));

    //ad-2
    m_pStartRedBtn = new QPushButton(tr("Infrared"));
    m_pStartRedBtn->setFixedWidth(130);
    connect(m_pStartRedBtn,SIGNAL(clicked()),this,SLOT(slotOnStartRed()));

    QLabel *pRedWidthLbl = new QLabel(tr("Infrared width(mm)"));
    pRedWidthLbl->setFixedWidth(80);
    m_pRedWidthDSpb = new QDoubleSpinBox;
    m_pRedWidthDSpb->setRange(0,100);
    m_pRedWidthDSpb->setDecimals(2);
    m_pRedWidthDSpb->setFixedWidth(130);
    m_pRedWidthDSpb->setValue(30.00);
    QLabel *pRedHeightLbl = new QLabel(tr("Infrared height(mm)"));
    pRedHeightLbl->setFixedWidth(80);
    m_pRedHeightDSpb = new QDoubleSpinBox;
    m_pRedHeightDSpb->setRange(0,100);
    m_pRedHeightDSpb->setDecimals(2);
    m_pRedHeightDSpb->setFixedWidth(130);
    m_pRedHeightDSpb->setValue(10.00);
    QLabel *pRedXLbl = new QLabel(tr("Infrared center x"));
    pRedXLbl->setFixedWidth(80);
    m_pRedXDSpb = new QDoubleSpinBox;
    m_pRedXDSpb->setRange(-100,100);
    m_pRedXDSpb->setDecimals(2);
    m_pRedXDSpb->setFixedWidth(130);
    m_pRedXDSpb->setValue(0.00);
    QLabel *pRedYLbl = new QLabel(tr("Infrared center y"));
    pRedYLbl->setFixedWidth(80);
    m_pRedYDSpb = new QDoubleSpinBox;
    m_pRedYDSpb->setRange(-100,100);
    m_pRedYDSpb->setDecimals(2);
    m_pRedYDSpb->setFixedWidth(130);
    m_pRedYDSpb->setValue(0.00);
    m_pStartRedWithParaBtn = new QPushButton(tr("Infrared size/pos"));
    m_pStartRedWithParaBtn->setFixedWidth(130);
    connect(m_pStartRedWithParaBtn,SIGNAL(clicked()),this,SLOT(slotOnStartRedWithPara()));
    m_pStopRedBtn = new QPushButton(tr("Stop infrared"));
    m_pStopRedBtn->setFixedWidth(130);
    connect(m_pStopRedBtn,SIGNAL(clicked()),this,SLOT(slotOnStop()));

    QWidget *ad2Widget = new QWidget;
    QGridLayout *pGltAd2 = new QGridLayout(ad2Widget);
    pGltAd2->addWidget(m_pStartRedBtn,0,1,1,1);
    pGltAd1->addItem(new QSpacerItem(100,30),1,1,1,1);
    pGltAd2->addWidget(pRedWidthLbl,2,0,1,1);
    pGltAd2->addWidget(m_pRedWidthDSpb,2,1,1,1);
    pGltAd2->addWidget(pRedHeightLbl,3,0,1,1);
    pGltAd2->addWidget(m_pRedHeightDSpb,3,1,1,1);
    pGltAd2->addWidget(pRedXLbl,4,0,1,1);
    pGltAd2->addWidget(m_pRedXDSpb,4,1,1,1);
    pGltAd2->addWidget(pRedYLbl,5,0,1,1);
    pGltAd2->addWidget(m_pRedYDSpb,5,1,1,1);
    pGltAd2->addWidget(m_pStartRedWithParaBtn,6,1,1,1);
    pGltAd2->addWidget(m_pStopRedBtn,7,1,1,1);

    m_pTabWidget->insertTab(1,ad2Widget,tr("Infrared"));

    //ad-3
    QLabel *pHoriLbl = new QLabel(tr("H offset"));
    pHoriLbl->setFixedWidth(80);
    m_pHoriDSpb = new QDoubleSpinBox;
    m_pHoriDSpb->setRange(-100,100);
    m_pHoriDSpb->setDecimals(1);
    m_pHoriDSpb->setValue(0.0);
    m_pHoriDSpb->setFixedWidth(130);
    QLabel *pVeriLbl = new QLabel(tr("V offset"));
    pVeriLbl->setFixedWidth(80);
    m_pVeriDSpb = new QDoubleSpinBox;
    m_pVeriDSpb->setRange(-100,100);
    m_pVeriDSpb->setDecimals(1);
    m_pVeriDSpb->setValue(0.0);
    m_pVeriDSpb->setFixedWidth(130);
    QLabel *pRotateLbl = new QLabel(tr("Angle"));
    pRotateLbl->setFixedWidth(80);
    m_pRotateDSpb = new QDoubleSpinBox;
    m_pRotateDSpb->setRange(-100,100);
    m_pRotateDSpb->setDecimals(1);
    m_pRotateDSpb->setValue(0.0);
    m_pRotateDSpb->setFixedWidth(130);
    m_pOffsetBtn = new QPushButton(tr("Transform mark"));
    m_pOffsetBtn->setFixedWidth(130);
    connect(m_pOffsetBtn,SIGNAL(clicked()),this,SLOT(slotOnDoOffset()));

    QLabel *pGroupLbl = new QLabel(tr("Group num"));
    pGroupLbl->setFixedWidth(80);
    m_pGroupSpb = new QSpinBox;
    m_pGroupSpb->setRange(0,20);
    m_pGroupSpb->setFixedWidth(130);
    m_pGroupBtn = new QPushButton(tr("Mark group"));
    m_pGroupBtn->setFixedWidth(130);
    connect(m_pGroupBtn,SIGNAL(clicked()),this,SLOT(slotOnDoGroup()));

    QLabel *pmarkNameLbl = new QLabel(tr("Entity name"));
    pmarkNameLbl->setFixedWidth(80);
    m_pMarkNameEdit = new QLineEdit;
    m_pMarkNameEdit->setFixedWidth(130);
    m_pMarkNameBtn = new QPushButton(tr("Mark specific entity"));
    m_pMarkNameBtn->setFixedWidth(130);
    connect(m_pMarkNameBtn,SIGNAL(clicked()),this,SLOT(slotOnDoMarkname()));

    QWidget *ad3Widget = new QWidget;
    QGridLayout *pGltAd3 = new QGridLayout(ad3Widget);
    pGltAd3->addWidget(pHoriLbl,0,0,1,1);
    pGltAd3->addWidget(m_pHoriDSpb,0,1,1,1);
    pGltAd3->addWidget(pVeriLbl,1,0,1,1);
    pGltAd3->addWidget(m_pVeriDSpb,1,1,1,1);
    pGltAd3->addWidget(pRotateLbl,2,0,1,1);
    pGltAd3->addWidget(m_pRotateDSpb,2,1,1,1);
    pGltAd3->addWidget(m_pOffsetBtn,3,1,1,1);
    pGltAd3->addItem(new QSpacerItem(100,30),4,1,1,1);
    pGltAd3->addWidget(pGroupLbl,5,0,1,1);
    pGltAd3->addWidget(m_pGroupSpb,5,1,1,1);
    pGltAd3->addWidget(m_pGroupBtn,6,1,1,1);
    pGltAd1->addItem(new QSpacerItem(100,30),7,1,1,1);
    pGltAd3->addWidget(pmarkNameLbl,8,0,1,1);
    pGltAd3->addWidget(m_pMarkNameEdit,8,1,1,1);
    pGltAd3->addWidget(m_pMarkNameBtn,9,1,1,1);

    m_pTabWidget->addTab(ad3Widget,tr("Advance 2"));

    m_pTabWidget->setEnabled(false);

    LabelLink *pLogoLbl = new LabelLink;
    pLogoLbl->setFixedSize(_LOGO_WIDTH,_LOGO_HEIGHT);
    QPixmap logoPm("logo.png");
    if(!logoPm.isNull()) {
        pLogoLbl->setPixmap(logoPm);
    }

    QVBoxLayout *pVlt = new QVBoxLayout;
    pVlt->addLayout(pDeviceHlt);
    pVlt->addWidget(_p_networkWgt);
    pVlt->addWidget(_p_serialportWgt);
    pVlt->addItem(new QSpacerItem(100,30));
    //pVlt->addLayout(pGlt);
    pVlt->addWidget(m_pTabWidget);
    pVlt->addWidget(pLogoLbl);
    pVlt->addStretch();

    QLabel *pLogLbl = new QLabel(tr("Log"));

    _p_logEdit = new QPlainTextEdit;
    _p_logEdit->setReadOnly(true);

    QLabel *pRawBytesLbl = new QLabel(tr("Debug data"));

    _p_rawBytesEdit = new QPlainTextEdit;
    _p_rawBytesEdit->setFixedHeight(45);

    _p_sendRawBytesBtn = new QPushButton(tr("Ascii send"));
    connect(_p_sendRawBytesBtn,SIGNAL(clicked()),this,SLOT(doSendRawBytes()));

    _p_sendHexRawBytesBtn = new QPushButton(tr("Hex send"));
    connect(_p_sendHexRawBytesBtn,SIGNAL(clicked()),this,SLOT(doSendHexRawBytes()));

    QVBoxLayout *pSendBtnVlt = new QVBoxLayout;
    pSendBtnVlt->addWidget(_p_sendRawBytesBtn);
    pSendBtnVlt->addWidget(_p_sendHexRawBytesBtn);

    QHBoxLayout *pRawBytesHlt = new QHBoxLayout;
    pRawBytesHlt->addWidget(_p_rawBytesEdit);
    pRawBytesHlt->addLayout(pSendBtnVlt);

    QVBoxLayout *pLogVlt = new QVBoxLayout;
    pLogVlt->addWidget(pLogLbl);
    pLogVlt->addWidget(_p_logEdit);
    pLogVlt->addWidget(pRawBytesLbl);
    pLogVlt->addLayout(pRawBytesHlt);

    QHBoxLayout *pHlt = new QHBoxLayout;
    pHlt->addLayout(pVlt);
    pHlt->addLayout(pLogVlt);
    pHlt->setStretchFactor(pLogVlt,2.0);

    setLayout(pHlt);
    setFixedSize(750,652);

    slotOnDeviceChanged();
}

Widget::~Widget()
{
}

// 返回当前使用的通讯接口（网口/串口）
CommIODevice *Widget::currentDevice() const
{
    CommIODevice *pDevice = 0;
    if(_p_networkWgt->isAvailable() && _p_networkWgt->canWrite()) {
        pDevice = _p_networkWgt;
    } else if(_p_serialportWgt->isAvailable() && _p_serialportWgt->canWrite()) {
        pDevice = _p_serialportWgt;
    }

    return pDevice;
}

void Widget::slotOnDeviceChanged()
{
    _p_networkWgt->setVisible(_p_networkRbn->isChecked());
    _p_serialportWgt->setVisible(_p_serialportRbn->isChecked());
}

void Widget::slotOnHasConnected()
{
    _p_serialportRbn->setEnabled(false);
    _p_networkRbn->setEnabled(false);

    m_pTabWidget->setEnabled(true);

    _p_logEdit->clear();
}

void Widget::slotOnHasDisconnected()
{
    _p_serialportRbn->setEnabled(true);
    _p_networkRbn->setEnabled(true);

    m_pTabWidget->setEnabled(false);
}

// 根据控件中写入的文档名，发送调用文档命令
void Widget::slotOnSwitchDoc()
{
    QString docName = _p_docEdit->text();
    if(docName.isEmpty()) { // 文档名不能为空提示
        QMessageBox::information(this,tr("Tip"),tr("Document name can not be empty."));
        return;
    }

    write("<L" + docName + ">");
}

// 根据控件中写入的标记名称和内容，发送修改标记内容命令
void Widget::slotOnChangeContent()
{
    QString name = _p_entityNameEdit->text();
    if(name.isEmpty()) { // 标记名称不能为空提示
        QMessageBox::information(this,tr("Tip"),tr("Entity name can not be empty"));
        return;
    }

    QString content = _p_entityContentEdit->text();
    if(content.isEmpty()) { // 内容不能为空提示
        QMessageBox::information(this,tr("Tip"),tr("Entity content can not be empty."));
        return;
    }

    write("<D" + name + "," + content + ">");
}

// 发送开始打标命令
void Widget::slotOnStart()
{
    write("<X>");
}

// 发送停止打标命令
void Widget::slotOnStop()
{
    write("<P>");
}

// 发送测试命令
void Widget::slotOnTest()
{
    QString name = m_pTestEdit->text();
    if(name.isEmpty()) {
        QMessageBox::information(this,tr("Tip"),tr("Test content can not be empty."));
        return;
    }
    write("<E" + name + ">");
}

// 根据控件的值，发送修改笔号参数命令
void Widget::slotOnChangePen()
{
    QString speed = m_pSpeedEdit->text(); // 获取笔号的打标速度
    if(speed.isEmpty()) { // 不能为空提示
        QMessageBox::information(this,tr("Tip"),tr("Speed can not be empty."));
        return;
    }

    QString power = m_pPowerEdit->text(); // 获取笔号的功率
    if(power.isEmpty()) { // 不能为空提示
        QMessageBox::information(this,tr("Tip"),tr("Power can not be empty."));
        return;
    }

    QString freq = m_pFreqEdit->text(); // 获取笔号的频率
    if(freq.isEmpty()) { // 不能为空提示
        QMessageBox::information(this,tr("Tip"),tr("Frequency can not be empty."));
        return;
    }

    // 三条命令一起发送，分别是修改笔号的打标速度命令，修改笔号的功率命令，修改笔号的频率命令
    write("<DSPEED," + speed + "><DPOWER,"+power+"><DFREQ,"+freq+">");
}

// 发送读取文档已打标个数命令
void Widget::slotOnReadMarkCount()
{
    write("<?MARKCOUNT>");
}

// 发送读取文档打标时间命令
void Widget::slotOnReadMarkTime()
{
    write("<?MARKTIME>");
}

// 发送开始红光预览命令
void Widget::slotOnStartRed()
{
    write("<XP>");
}

// 发送指定大小位置的开始红光预览命令
void Widget::slotOnStartRedWithPara()
{
    QString redWidth = QString::number(m_pRedWidthDSpb->value(),'f',2); // 红光预览宽度
    QString redHeight = QString::number(m_pRedHeightDSpb->value(),'f',2);  // 红光预览高度
    QString redX = QString::number(m_pRedXDSpb->value(),'f',2); // 红光预览中心位置X值
    QString redY = QString::number(m_pRedYDSpb->value(),'f',2); // 红光预览中心位置Y值
    QString sendArr = QString("<XP,%1,%2,%3,%4>")
            .arg(redWidth).arg(redHeight).arg(redX).arg(redY);
    write(sendArr);
}

// 发送偏移旋转打标命令
void Widget::slotOnDoOffset()
{
    QString hori = QString::number(m_pHoriDSpb->value(),'f',1); // 水平偏移值
    QString veri = QString::number(m_pVeriDSpb->value(),'f',1); // 竖直偏移值
    QString rotate = QString::number(m_pRotateDSpb->value(),'f',1); // 旋转度数
    write(QString("<T,3,%1,%2,%3><X>").arg(hori).arg(veri).arg(rotate));
}

// 发送分组标签打标命令
void Widget::slotOnDoGroup()
{
    write(QString("<S%1,X>").arg(QString::number(m_pGroupSpb->value())));
}

// 发送选择标记名直接打标命令
void Widget::slotOnDoMarkname()
{
    QString markname = m_pMarkNameEdit->text();
    if(markname.isEmpty()) {
        QMessageBox::information(this,tr("Tip"),tr("Entity name can not be empty."));
        return;
    }
    write(QString("<C%1,X>").arg(markname));
}

// 从当前接口读取数据
void Widget::doRead()
{
    // 获取当前使用的通讯接口（网口/串口）
    CommIODevice *pDevice = currentDevice();
    if(!pDevice) {
        return;
    }

    // 读取全部数据并显示在日志控件上
    readToLog(pDevice->readAll());
}

// ASCII字符发送，则直接发送数据控件中的内容
void Widget::doSendRawBytes()
{
    write(_p_rawBytesEdit->toPlainText());
}

// 十六进制发送，则将十六进制转换成字符后发送
void Widget::doSendHexRawBytes()
{
    QByteArray bytes = QByteArray::fromHex(_p_rawBytesEdit->toPlainText().toUtf8());
    if(bytes.isEmpty()) {
        return;
    }

    write(bytes);
}

// 发送数据到板卡
int Widget::write(const QString &content)
{
    // 获取当前使用的通讯接口（网口/串口）
    CommIODevice *pDevice = currentDevice();
    if(!pDevice) {
        return -1;
    }

    if(content.isEmpty()) {
        return -1;
    }

    QByteArray bytes;

    // 根据通讯模式，转换数据格式为ASCII/Unicode
    // ascii
    if(pDevice->protocol() == 0) {
        bytes = content.toLatin1();
    }

    // unicode
    else {
        for(int i = 0;i < content.size();i++) {
            ushort code = content.at(i).unicode();
            bytes.append((char)((code >> 8) & 0xFF));
            bytes.append((char)(code & 0xFF));
        }
    }


    // 将发送数据显示在日志控件上
    writeToLog(content,bytes);

    // 发送数据
    return pDevice->write(bytes);
}

void Widget::writeToLog(const QString &text,const QByteArray &bytes)
{
    if(bytes.isEmpty()) { return; }

    _p_logEdit->appendPlainText(QTime::currentTime().toString(tr("hh:mm:ss")));
    _p_logEdit->appendPlainText(QString(tr("Send(text): ")) + text);
    _p_logEdit->appendPlainText(QString(tr("Send(hex ): ")) + toHex(bytes));
    _p_logEdit->appendPlainText(QString(" "));
}

void Widget::readToLog(const QByteArray &bytes)
{
    CommIODevice *pDevice = currentDevice();
    if(!pDevice) {
        return;
    }

    if(bytes.isEmpty()) { return; }

    QString text;
     if(pDevice->protocol() == 0) {
         text = QString(bytes);
     } else {
         int count = bytes.size() / 2;
         for(int i = 0;i < count;i++) {
             uchar hByte = (uchar)bytes.at(i * 2);
             uchar lByte = (uchar)bytes.at(i * 2 + 1);
             text.append(QChar((ushort)((hByte << 8) | lByte)));
         }
     }

    _p_logEdit->appendPlainText(QTime::currentTime().toString(tr("hh:mm:ss")));
    _p_logEdit->appendPlainText(QString(tr("Recv(text): ")) + text);
    _p_logEdit->appendPlainText(QString(tr("Recv(hex ): ")) + toHex(bytes));
    _p_logEdit->appendPlainText(QString(" "));
}
