#ifndef WIDGET_H
#define WIDGET_H

#include <QWidget>
#include <QTcpSocket>
#include <QLabel>
#include <QLineEdit>
#include <QTime>

class QRadioButton;
class QLineEdit;
class QPushButton;
class QComboBox;
class QPlainTextEdit;
class NetworkWdiget;
class SerialPortWidget;
class CommIODevice;
class QTabWidget;
class QDoubleSpinBox;
class QSpinBox;

// show with image and link the web
class LabelLink : public QLabel
{
    Q_OBJECT
public:
    LabelLink(QWidget *parent = 0);

    void setEnableClickInfo(bool on);

protected:
    void mousePressEvent(QMouseEvent *event);
    void resizeEvent(QResizeEvent *event);
    void mouseMoveEvent(QMouseEvent *event);

private:
    bool _clickInfo;
    QTime _prevTime;
    int _repeatCount;
};


//
class LineEdit : public QLineEdit
{
    Q_OBJECT
public:
    LineEdit(QWidget *parent = 0);

protected:
    virtual void focusInEvent(QFocusEvent *e);
    virtual void focusOutEvent(QFocusEvent *e);
    virtual void inputMethodEvent(QInputMethodEvent *e);
};


// 主界面
class Widget : public QWidget
{
    Q_OBJECT

public:
    Widget(QWidget *parent = 0);
    ~Widget();

private:

protected slots:
    void slotOnDeviceChanged();

    void slotOnHasConnected();
    void slotOnHasDisconnected();

    void slotOnSwitchDoc();
    void slotOnChangeContent();
    void slotOnStart();
    void slotOnStop();

    void slotOnTest();
    void slotOnChangePen();

    void slotOnReadMarkCount();
    void slotOnReadMarkTime();

    void slotOnStartRed();
    void slotOnStartRedWithPara();

    void slotOnDoOffset();

    void slotOnDoGroup();
    void slotOnDoMarkname();

protected slots:
    void doRead();
    void doSendRawBytes();
    void doSendHexRawBytes();

private:
    CommIODevice *currentDevice() const;

    int write(const QString &content);

    void writeToLog(const QString &text,const QByteArray &bytes);
    void readToLog(const QByteArray &bytes);

    QRadioButton *_p_serialportRbn;
    QRadioButton *_p_networkRbn;

    NetworkWdiget *_p_networkWgt;

    SerialPortWidget *_p_serialportWgt;

    QLineEdit *_p_docEdit;
    QPushButton *_p_docSwitchBtn;

    LineEdit *_p_entityNameEdit;
    LineEdit *_p_entityContentEdit;
    QPushButton *_p_changeBtn;

    QPushButton *_p_startBtn;
    QPushButton *_p_stopBtn;

    QPlainTextEdit *_p_logEdit;

    QPlainTextEdit *_p_rawBytesEdit;
    QPushButton *_p_sendRawBytesBtn;
    QPushButton *_p_sendHexRawBytesBtn;

    QTcpSocket *_p_socket;

    QTabWidget *m_pTabWidget;

    QLineEdit *m_pTestEdit;
    QPushButton *m_pTestBtn;

    QLineEdit *m_pSpeedEdit;
    QLineEdit *m_pPowerEdit;
    QLineEdit *m_pFreqEdit;
    QPushButton *m_pChangedBtn;

    QPushButton *m_pMarkCountBtn;
    QPushButton *m_pMarkTimeBtn;

    QPushButton *m_pStartRedBtn;
    QPushButton *m_pStopRedBtn;

    QDoubleSpinBox *m_pRedWidthDSpb;
    QDoubleSpinBox *m_pRedHeightDSpb;
    QDoubleSpinBox *m_pRedXDSpb;
    QDoubleSpinBox *m_pRedYDSpb;
    QPushButton *m_pStartRedWithParaBtn;

    QDoubleSpinBox *m_pHoriDSpb;
    QDoubleSpinBox *m_pVeriDSpb;
    QDoubleSpinBox *m_pRotateDSpb;
    QPushButton *m_pOffsetBtn;

    QSpinBox *m_pGroupSpb;
    QPushButton *m_pGroupBtn;

    QLineEdit *m_pMarkNameEdit;
    QPushButton *m_pMarkNameBtn;
};

#endif // WIDGET_H
